/*	libljsdcdev.c -- ljsdcdev
	libljsdcdev funtions
	ljsdcdev
	Copyright (C) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljsdcdev.h"

int ljsdcdev_random(int min, int max)	{
	return randnum(max - min + 1) + min;
}

void ljsdcdev_line_tr(float x1, float y1, float x2, float y2, uint8 a, uint8 r, uint8 g, uint8 b)	{
	pvr_poly_hdr_t hdr;
	pvr_poly_cxt_t cxt;
	pvr_vertex_t vert;

	pvr_poly_cxt_col(&cxt, PVR_LIST_OP_POLY);
	pvr_poly_compile(&hdr, &cxt);
	pvr_prim(&hdr, sizeof(hdr));

	vert.flags = PVR_CMD_VERTEX;
	vert.x = x1 - 1;
	vert.y = y1;
	vert.z = 5.0f;
	vert.u = 0;
	vert.v = 0;
	vert.argb = PVR_PACK_COLOR(a / 255, r / 255, g / 255, b / 255);
	vert.oargb = 0;
	pvr_prim(&vert, sizeof(vert));

	vert.x = x2  - 1;
	vert.y = y2;
	pvr_prim(&vert, sizeof(vert));

	vert.x = x1 + 1;
	vert.y = y1;
	pvr_prim(&vert, sizeof(vert));

	vert.flags = PVR_CMD_VERTEX_EOL;
	vert.x = x2 + 1;
	vert.y = y2;
	pvr_prim(&vert, sizeof(vert));
}

void ljsdcdev_hline_tr(float x1, float x2, float y, uint8 a, uint8 r, uint8 g, uint8 b)	{
	pvr_poly_hdr_t hdr;
	pvr_poly_cxt_t cxt;
	pvr_vertex_t vert;
	float cx;
	if(x1 > x2)	{
		cx = x1;
		x1 = x2;
		x2 = cx;
	}

	pvr_poly_cxt_col(&cxt, PVR_LIST_OP_POLY);
	pvr_poly_compile(&hdr, &cxt);

	pvr_prim(&hdr, sizeof(hdr));
	vert.flags = PVR_CMD_VERTEX;
	vert.x = x1;
	vert.y = y + 1;
	vert.z = 5.0f;
	vert.u = 0;
	vert.v = 0;
	vert.argb = PVR_PACK_COLOR(a / 255, r / 255, g / 255, b / 255);
	vert.oargb = 0;
	pvr_prim(&vert, sizeof(vert));

	vert.y = y - 1;
	pvr_prim(&vert, sizeof(vert));

	vert.x = x2;
	vert.y = y + 1;
	pvr_prim(&vert, sizeof(vert));

	vert.flags = PVR_CMD_VERTEX_EOL;
	vert.y = y - 1;
	pvr_prim(&vert, sizeof(vert));
}

void ljsdcdev_vline_tr(float x, float y1, float y2, uint8 a, uint8 r, uint8 g, uint8 b)	{
	pvr_poly_hdr_t hdr;
	pvr_poly_cxt_t cxt;
	pvr_vertex_t vert;
	float cy;
	if(y1 > y2)	{
		cy = y1;
		y1 = y2;
		y2 = cy;
	}

	pvr_poly_cxt_col(&cxt, PVR_LIST_OP_POLY);
	pvr_poly_compile(&hdr, &cxt);

	pvr_prim(&hdr, sizeof(hdr));
	vert.flags = PVR_CMD_VERTEX;
	vert.x = x  - 1;
	vert.y = y2;
	vert.z = 5.0f;
	vert.u = 0;
	vert.v = 0;
	vert.argb = PVR_PACK_COLOR(a / 255, r / 255, g / 255, b / 255);
	vert.oargb = 0;
	pvr_prim(&vert, sizeof(vert));

	vert.y = y1;
	pvr_prim(&vert, sizeof(vert));

	vert.x = x + 1;
	vert.y = y2;
	pvr_prim(&vert, sizeof(vert));

	vert.flags = PVR_CMD_VERTEX_EOL;
	vert.y = y1;
	pvr_prim(&vert, sizeof(vert));
}

void ljsdcdev_arrow(float x1, float y1, float x2, float y2, float x3, float y3, uint8 a, uint8 r, uint8 g, uint8 b)	{
	pvr_poly_hdr_t hdr;
	pvr_poly_cxt_t cxt;
	pvr_vertex_t vert;

	pvr_poly_cxt_col(&cxt, PVR_LIST_OP_POLY);
	pvr_poly_compile(&hdr, &cxt);
	pvr_prim(&hdr, sizeof(hdr));

	vert.flags = PVR_CMD_VERTEX;
	vert.x = x1;
	vert.y = y1;
	vert.z = 5.0f;
	vert.u = 0;
	vert.v = 0;
	vert.argb = PVR_PACK_COLOR(a / 255, r / 255, g / 255, b / 255);
	vert.oargb = 0;
	pvr_prim(&vert, sizeof(vert));

	vert.x = x2;
	vert.y = y2;
	pvr_prim(&vert, sizeof(vert));

	vert.flags = PVR_CMD_VERTEX_EOL;
	vert.x = x3;
	vert.y = y3;
	pvr_prim(&vert, sizeof(vert));
}

void ljsdcdev_text(plx_fcxt_t * fcxt, const char * msg, float x, float y, uint8 a, uint8 r, uint8 g, uint8 b)	{
	plx_fcxt_begin(fcxt);
	plx_fcxt_setpos(fcxt, x, y, 5.0f);
	plx_fcxt_setcolor4f(fcxt, a / 255.0f, r / 255.0f, g / 255.0f, b / 255.0f);
	plx_fcxt_draw(fcxt, msg);
	plx_fcxt_end(fcxt);
}
