/*	main.c -- ljsdcdev
	Program startup, and other stuff through opening menu
	ljsdcdev
	Copyright (C) 2002, 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljsdcdev.h"

static inline void setup_magics()	{
	xml_parse_type(1);
}

static inline void setup_skills()	{
	xml_parse_type(2);
}

static inline void setup_items()	{
	xml_parse_type(0);
	itemlist[0].number = 1;
	itemlist[0].count = 5;
}

static inline void setup_equip()	{
	xml_parse_type(3);
	equiplist[0] = 1;
	equiplist[1] = 1;
}

static inline void setup_enemies()	{
	xml_parse_type(4);
}

static inline void setup_credits()	{
	xml_parse_type(5);
}

static void draw_discl()	{
	plx_fcxt_begin(fcxt_ax);
	plx_fcxt_setsize(fcxt_ax, 32.0f);
	plx_fcxt_setpos(fcxt_ax, 200.0f, 32.0f, 5.0f);
	plx_fcxt_draw(fcxt_ax, "Disclaimer");
	plx_fcxt_setsize(fcxt_ax, 22.0f);
	plx_fcxt_setpos(fcxt_ax, 16.0f, 64.0f, 5.0f);
	plx_fcxt_draw(fcxt_ax, "The ljsdcdev project comes with\n"
					"ABSOLUTELY NO WARRANTY and is\n"
					"freely distributed under the\n"
					"terms of the GNU General Public\n"
					"license, see the file COPYING\n"
					"for more information.\n\n"
					"ljsdcdev is in no way associated\n"
					"with or endorsed by Sega.\n\n"
					"The ljsdcdev Project is\n"
					"Copyright (C) 2002, 2003 Lawrence Sebald\n\n"
					"Visit http://ljsdcdev.sunsite.dk/\n"
					"or http://ljsdcdev.sf.net/\n"
					"for more information.\n\n"
					"Press Start to continue.");
	plx_fcxt_end(fcxt_ax);
}

int main(int argc, char * argv[])	{
	cont_state_t * state;
	plx_texture_t * bgimage;

	pvr_init_defaults();
	plx_dr_init(&dr_state);
	font_ax = plx_font_load("/cd/fonts/axaxax.txf");
	fcxt_ax = plx_fcxt_create(font_ax, PLX_LIST_TR_POLY);

	bgimage = plx_txr_load("/cd/images/discl.png", 0, PVR_TXRLOAD_16BPP);

	while(1)	{
		state = (cont_state_t *) maple_dev_status(maple_enum_type(0, MAPLE_FUNC_CONTROLLER));

		if(state != NULL)	{
			if(state->buttons & CONT_START)
				break;
		}

		plx_scene_begin();
		plx_list_begin(PLX_LIST_OP_POLY);
		plx_txr_send_hdr(bgimage, PLX_LIST_OP_POLY, 0);
		plx_spr_fnd(&dr_state, 640.0f, 480.0f, 320.0f, 240.0f, 5.0f, 1.0f, 1, 1, 1);
		plx_list_begin(PLX_LIST_TR_POLY);
		draw_discl(&dr_state);
		plx_scene_end();
	}

	time_t sttime = time(NULL);
	plx_txr_destroy(bgimage);

	bgimage = plx_txr_load("/cd/images/ljsdcdev.png", 0, PVR_TXRLOAD_16BPP);
	while(sttime + 5 > time(NULL))	{
		plx_scene_begin();
		plx_list_begin(PLX_LIST_OP_POLY);
		plx_txr_send_hdr(bgimage, PLX_LIST_OP_POLY, 0);
		plx_spr_fnd(&dr_state, 640.0f, 480.0f, 320.0f, 240.0f, 5.0f, 1.0f, 1, 1, 1);
		plx_scene_end();
	}

	plx_txr_destroy(bgimage);

	bgimage = plx_txr_load("/cd/images/menu.png", 0, PVR_TXRLOAD_16BPP);
	uint64 msecs = timer_ms_gettime64();
	float alpha = 1.0f;
	uint8 direction = 0;
	uint8 startpressed = 0;
	pvr_poly_hdr_t hdr;
	pvr_poly_cxt_t cxt;

	pvr_poly_cxt_col(&cxt, PVR_LIST_TR_POLY);
	pvr_poly_compile(&hdr, &cxt);
	pvr_vertex_t vert;
	uint8 sel = 0;

	setup_magics();
	setup_items();
	setup_equip();
	setup_skills();
	setup_enemies();
	setup_credits();

	plx_fcxt_setsize(fcxt_ax, 22.0f);

	while(1)	{
		state = (cont_state_t *) maple_dev_status(maple_enum_type(0, MAPLE_FUNC_CONTROLLER));

		if(startpressed == 0)	{
			if(state != NULL)	{
				if(state->buttons & CONT_START)	{
					startpressed = 1;
					thd_sleep(200);
					state = (cont_state_t *) maple_dev_status(maple_enum_type(0, MAPLE_FUNC_CONTROLLER));
				}
			}

			if(direction == 0 && msecs + 26 < timer_ms_gettime64())	{
				alpha -= 0.01f;
				if(alpha < 0.1f)	{
					direction = 1;
				}
				msecs = timer_ms_gettime64();
			}
			else if(direction == 1 && msecs + 26 < timer_ms_gettime64())	{
				alpha += 0.01f;
				if(alpha > 0.99f)	{
					direction = 0;
				}
				msecs = timer_ms_gettime64();
			}
			plx_scene_begin();
			plx_list_begin(PLX_LIST_OP_POLY);
			plx_txr_send_hdr(bgimage, PLX_LIST_OP_POLY, 0);
			plx_spr_fnd(&dr_state, 640.0f, 480.0f, 320.0f, 240.0f, 5.0f, 1.0f, 1, 1, 1);
			plx_list_begin(PLX_LIST_TR_POLY);
			plx_fcxt_begin(fcxt_ax);
			plx_fcxt_setcolor4f(fcxt_ax, 1.0f, 1, 1, 1);
			plx_fcxt_setpos(fcxt_ax, 150.0f, 100.0f, 5.0f);
			plx_fcxt_draw(fcxt_ax, "The ljsdcdev Project");
			plx_fcxt_setcolor4f(fcxt_ax, alpha, 1, 1, 1);
			plx_fcxt_setpos(fcxt_ax, 216.0f, 256.0f, 5.0f);
			plx_fcxt_draw(fcxt_ax, "Press Start");
			plx_fcxt_end(fcxt_ax);
			plx_scene_end();
		}
		else	{
			if(state != NULL)	{
				if(state->buttons & CONT_DPAD_UP)	{
					if(sel != 0)
						sel--;
					else
						sel = 2;
					thd_sleep(100);
				}
				if(state->buttons & CONT_DPAD_DOWN)	{
					if(sel != 2)
						sel++;
					else
						sel = 0;
					thd_sleep(100);
				}
				if(state->buttons & CONT_A)	{
					if(sel == 0)	{
						misc_stats_setup();
						plx_txr_destroy(bgimage);
						name_set(0);
						thd_sleep(300);
						name_set(1);
						thd_sleep(300);
						name_set(2);
						thd_sleep(300);
						name_set(3);
						thd_sleep(300);
						do_tilemap();
					}
					else if(sel == 1)	{
						if(load_game() == 0)	{
							plx_txr_destroy(bgimage);
							tilemap_init(posx, posy);
							misc_start_timer();
							tilemap_mainloop();
						}
						else	{
							plx_fcxt_setsize(fcxt_ax, 22.0f);
						}
					}
					else if(sel == 2)	{
						thd_sleep(200);
						credits_draw_screen();
					}
				}
			}

			plx_scene_begin();
			plx_list_begin(PLX_LIST_OP_POLY);
			plx_txr_send_hdr(bgimage, PLX_LIST_OP_POLY, 0);
			plx_spr_fnd(&dr_state, 640.0f, 480.0f, 320.0f, 240.0f, 5.0f, 1.0f, 1, 1, 1);
			plx_list_begin(PLX_LIST_TR_POLY);

			pvr_prim(&hdr, sizeof(hdr));
			vert.flags = PVR_CMD_VERTEX;
			vert.x = 200.0f;
			vert.y = 296.0f;
			vert.z = 5.0f;
			vert.u = 0.0f;
			vert.v = 0.0f;
			vert.argb = PVR_PACK_COLOR(0.5f, 0.0f, 0.0f, 1.0f);
			vert.oargb = 0;
			pvr_prim(&vert, sizeof(vert));

			vert.x = 200.0f;
			vert.y = 224.0f;
			pvr_prim(&vert, sizeof(vert));

			vert.x = 400.0f;
			vert.y = 296.0f;
			pvr_prim(&vert, sizeof(vert));

			vert.flags = PVR_CMD_VERTEX_EOL;
			vert.x = 400.0f;
			vert.y = 224.0f;
			pvr_prim(&vert, sizeof(vert));

			pvr_prim(&hdr, sizeof(hdr));
			vert.flags = PVR_CMD_VERTEX;
			vert.x = 200.0f;
			vert.y = 248 + (sel * 24);
			vert.argb = PVR_PACK_COLOR(0.5f, 0.0, 0.8f, 1.0f);
			pvr_prim(&vert, sizeof(vert));

			vert.y = 224 + (sel * 24);
			pvr_prim(&vert, sizeof(vert));

			vert.x = 400.0f;
			vert.y = 248 + (sel * 24);
			pvr_prim(&vert, sizeof(vert));

			vert.flags = PVR_CMD_VERTEX_EOL;
			vert.y = 224 + (sel * 24);
			pvr_prim(&vert, sizeof(vert));

			plx_fcxt_begin(fcxt_ax);
			plx_fcxt_setcolor4f(fcxt_ax, 1.0f, 1, 1, 1);
			plx_fcxt_setpos(fcxt_ax, 150.0f, 100.0f, 5.0f);
			plx_fcxt_setsize(fcxt_ax, 22.0f);
			plx_fcxt_draw(fcxt_ax, "The ljsdcdev Project");
			plx_fcxt_setpos(fcxt_ax, 216.0f, 248.0f, 5.0f);
			plx_fcxt_draw(fcxt_ax, "New Game\n"
							"Load Game\n"
							"Credits");
			plx_fcxt_end(fcxt_ax);
			plx_scene_end();
		}
	}
	plx_fcxt_destroy(fcxt_ax);
	pvr_shutdown();
	return 0;
}
