/*	misc.c -- ljsdcdev
	Misc. stuff that doesn't deserve its own file, but is still important
	ljsdcdev
	Copyright (C) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljsdcdev.h"

unsigned long misc_get_filesize(FILE * file)	{
	unsigned long size;

	fseek(file, 0, SEEK_SET);
	fseek(file, 0, SEEK_END);
	size = ftell(file);
	fseek(file, 0, SEEK_SET);

	return size;
}

int misc_get_itcount()	{
	int i, count = 0;
	for(i = 0; i < 40; i++)	{
		if(itemlist[i].number != 0)	{
			count++;
		}
	}
	return count;
}

int misc_get_eqcount()	{
	int i, count = 0;
	for(i = 0; i < 40; i++)	{
		if(equiplist[i] != 0)	{
			count++;
		}
	}
	return count;
}

static int __compare1(const uint8 * first, const uint8 * second)	{
	if(*first == *second)	{
		return 0;
	}
	else if(*first == 0)	{
		return 1;
	}
	else if(*second == 0)	{
		return -1;
	}
	else if(*first < *second)	{
		return -1;
	}
	else	{
		return 1;
	}
}

static int __compare2(const item_t * first, const item_t * second)	{
	if(first->number == second->number)	{
		return 0;
	}
	else if(first->number == 0)	{
		return 1;
	}
	else if(second->number == 0)	{
		return -1;
	}
	else if(first->number < second->number)	{
		return -1;
	}
	else	{
		return 1;
	}
}

void misc_sort_equip()	{
	qsort(equiplist, 40, sizeof(uint8), (int (*)(const void *, const void *)) __compare1);
}

void misc_sort_items()	{
	qsort(itemlist, 40, sizeof(item_t), (int (*)(const void *, const void *)) __compare2);
}

int misc_add_equip(int number)	{
	int i;
	for(i = 0; i < 40; i++)	{
		if(equiplist[i] == 0)	{
			equiplist[i] = number;
			return 0;
		}
	}

	return 1;
}

int misc_add_item(int number, int count)	{
	int i;
	for(i = 0; i < 40; i++)	{
		if(itemlist[i].number == 0 || itemlist[i].number == number)	{
			itemlist[i].number = number;
			itemlist[i].count += count;
			return 0;
		}
	}

	return 1;
}

void misc_start_timer()	{
	sttime = time(NULL);
}

void misc_update_timer()	{
	time_t current = time(NULL);
	gtime += current - sttime;
	sttime = current;
}

game_time_t misc_get_timer()	{
 	game_time_t now;
	now.h = gtime / 3600;
	now.m = (gtime % 3600) / 60;
	now.s = (gtime % 3600) % 60;

	return now;
}

int misc_check_skilldep(int ch, int type, int i)	{
	int j, k;
	uint16 * ptr;

	if(type == 0)	{
		ptr = stats[ch].skills.sk_type1;
	}
	else if(type == 1)	{
		ptr = stats[ch].skills.sk_type2;
	}
	else if(type == 2)	{
		ptr = stats[ch].skills.sk_type3;
	}
	else if(type == 3)	{
		ptr = stats[ch].skills.sk_type4;
	}
	else	{
		ptr = stats[ch].skills.sk_type5;
	}

	for(j = 0; j < 10; j++)	{
		if(ptr[j] == i + 1)	{
			return -1;
		}
	}

	for(k = 0; k < skills[i].req_count; k++)	{
		for(j = 0; j < 10; j++)	{
			if(ptr[j] == skills[i].req[k] + 1)	{
				break;
			}
			if(j == 9)	{
				return -1;
			}
		}
	}
	return 0;
}

int misc_add_skill(int num, int ch, int type)	{
	uint16 * ptr;
	uint8 * ptr2;
	uint8 * ptr3;
	int i;

	if(type == 0)	{
		ptr = stats[ch].skills.sk_type1;
		ptr2 = stats[ch].skills.sk_t1_cht;
		ptr3 = stats[ch].skills.sk_t1_chl;
	}
	else if(type == 1)	{
		ptr = stats[ch].skills.sk_type2;
		ptr2 = stats[ch].skills.sk_t2_cht;
		ptr3 = stats[ch].skills.sk_t2_chl;
	}
	else if(type == 2)	{
		ptr = stats[ch].skills.sk_type3;
		ptr2 = stats[ch].skills.sk_t3_cht;
		ptr3 = stats[ch].skills.sk_t3_chl;
	}
	else if(type == 3)	{
		ptr = stats[ch].skills.sk_type4;
		ptr2 = stats[ch].skills.sk_t4_cht;
		ptr3 = stats[ch].skills.sk_t4_chl;
	}
	else	{
		ptr = stats[ch].skills.sk_type5;
		ptr2 = stats[ch].skills.sk_t5_cht;
		ptr3 = stats[ch].skills.sk_t5_chl;
	}

	for(i = 0; i < 10; i++)	{
		if(ptr[i] == 0)	{
			ptr[i] = num;
			ptr2[i] = 1;
			ptr3[i] = 1;
			return 0;
		}
	}

	return -1;
}

void misc_stats_setup()	{
	int i, j;
	strcpy(stats[0].name, "L.J.");
	strcpy(stats[1].name, "Jon");
	strcpy(stats[2].name, "T.C.");
	strcpy(stats[3].name, "Nick");

	stats[0].stats.hp = 30;
	stats[0].stats.hp_rem = 30;
	stats[0].stats.attack = 9;
	stats[0].stats.speed = 9;
	stats[0].stats.defense = 7;
	stats[0].stats.strength = 6;

	stats[1].stats.hp = 25;
	stats[1].stats.hp_rem = 25;
	stats[1].stats.attack = 8;
	stats[1].stats.speed = 8;
	stats[1].stats.defense = 7;
	stats[1].stats.strength = 5;

	stats[2].stats.hp = 20;
	stats[2].stats.hp_rem = 20;
	stats[2].stats.speed = 7;
	stats[2].stats.attack = 7;
	stats[2].stats.defense = 6;
	stats[2].stats.matt = 5;
	stats[2].stats.strength = 2;

	stats[3].stats.hp = 21;
	stats[3].stats.hp_rem = 21;
	stats[3].stats.attack = 8;
	stats[3].stats.defense = 6;
	stats[3].stats.speed = 6;
	stats[3].stats.matt = 6;
	stats[3].stats.strength = 1;

	stats[0].stats.mp = 0;
	stats[0].stats.mp_rem = 0;

	stats[1].stats.mp = 0;
	stats[1].stats.mp_rem = 0;

	stats[2].stats.mp = 10;
	stats[2].stats.mp_rem = 10;

	stats[3].stats.mp = 10;
	stats[3].stats.mp_rem = 10;

	stats[0].ally = stats[1].ally = stats[2].ally = stats[3].ally = 1;

	stats[0].stats.rank = 0;
	stats[1].stats.rank = 1;
	stats[2].stats.rank = 2;
	stats[3].stats.rank = 3;

	stats[0].stats.level = 1;
	stats[1].stats.level = 1;
	stats[2].stats.level = 1;
	stats[3].stats.level = 1;

	for(i = 0; i < 4; i++)	{
		stats[0].magic.mag_lev1[i] = 0;
		stats[0].magic.mag_lev2[i] = 0;
		stats[0].magic.mag_lev3[i] = 0;
		stats[0].magic.mag_lev4[i] = 0;
		stats[0].magic.mag_lev5[i] = 0;
		stats[0].magic.mag_lev6[i] = 0;
		stats[0].magic.mag_lev7[i] = 0;
		stats[0].magic.mag_lev8[i] = 0;
		stats[0].magic.mag_lev9[i] = 0;
		stats[0].magic.mag_lev10[i] = 0;
		stats[1].magic.mag_lev1[i] = 0;
		stats[1].magic.mag_lev2[i] = 0;
		stats[1].magic.mag_lev3[i] = 0;
		stats[1].magic.mag_lev4[i] = 0;
		stats[1].magic.mag_lev5[i] = 0;
		stats[1].magic.mag_lev6[i] = 0;
		stats[1].magic.mag_lev7[i] = 0;
		stats[1].magic.mag_lev8[i] = 0;
		stats[1].magic.mag_lev9[i] = 0;
		stats[1].magic.mag_lev10[i] = 0;
		stats[2].magic.mag_lev1[i] = 0;
		stats[2].magic.mag_lev2[i] = 0;
		stats[2].magic.mag_lev3[i] = 0;
		stats[2].magic.mag_lev4[i] = 0;
		stats[2].magic.mag_lev5[i] = 0;
		stats[2].magic.mag_lev6[i] = 0;
		stats[2].magic.mag_lev7[i] = 0;
		stats[2].magic.mag_lev8[i] = 0;
		stats[2].magic.mag_lev9[i] = 0;
		stats[2].magic.mag_lev10[i] = 0;
		stats[3].magic.mag_lev1[i] = 0;
		stats[3].magic.mag_lev2[i] = 0;
		stats[3].magic.mag_lev3[i] = 0;
		stats[3].magic.mag_lev4[i] = 0;
		stats[3].magic.mag_lev5[i] = 0;
		stats[3].magic.mag_lev6[i] = 0;
		stats[3].magic.mag_lev7[i] = 0;
		stats[3].magic.mag_lev8[i] = 0;
		stats[3].magic.mag_lev9[i] = 0;
		stats[3].magic.mag_lev10[i] = 0;
	}

	for(i = 0; i < 10; i++)	{
		for(j = 0; j < 4; j++)	{
			stats[j].equip.equipment[i] = 0;
		}
	}

	for(i = 0; i < 10; i++)	{
		for(j = 0; j < 4; j++)	{
			stats[j].skills.sk_type1[i] = 0;
			stats[j].skills.sk_t1_chl[i] = 0;
			stats[j].skills.sk_t1_cht[i] = 0;
			stats[j].skills.sk_type2[i] = 0;
			stats[j].skills.sk_t2_chl[i] = 0;
			stats[j].skills.sk_t2_cht[i] = 0;
			stats[j].skills.sk_type3[i] = 0;
			stats[j].skills.sk_t3_chl[i] = 0;
			stats[i].skills.sk_t3_cht[i] = 0;
			stats[j].skills.sk_type4[i] = 0;
			stats[j].skills.sk_t4_chl[i] = 0;
			stats[j].skills.sk_t4_cht[i] = 0;
			stats[j].skills.sk_type5[i] = 0;
			stats[j].skills.sk_t5_chl[i] = 0;
			stats[j].skills.sk_t5_cht[i] = 0;
		}
	}

	stats[0].equip.equipment[0] = 1;

	stats[0].images.splayer = plx_txr_load("/cd/images/fighter.png", 1, PVR_TXRLOAD_16BPP);
	stats[1].images.splayer = plx_txr_load("/cd/images/thief.png", 1, PVR_TXRLOAD_16BPP);
	stats[2].images.splayer = plx_txr_load("/cd/images/bkmage.png", 1, PVR_TXRLOAD_16BPP);
	stats[3].images.splayer = plx_txr_load("/cd/images/wtmage.png", 1, PVR_TXRLOAD_16BPP);

	options.mc.r = 0x00;
	options.mc.g = 0x00;
	options.mc.b = 0x00;

	options.tc.r = 0xFF;
	options.tc.g = 0xFF;
	options.tc.b = 0xFF;

	options.bc.r = 0xFF;
	options.bc.g = 0xFF;
	options.bc.b = 0xFF;
	gtime = 0;
}
