/*	plxljm2.c -- ljsdcdev
	.ljm playback library for DC
	ljsdcdev
	Copyright (C) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljm.h"

/* Loads a ljm map, and fills in the information required about it */
/* This really should have some sort of error protection, but none here for right now. */
int ljm_load(const char * fn, int magnify)	{
	uint32 i;
	char * tmp_txr;
	FILE * fp;
	uint64 starttime;
	char sig[3];

	fp = fopen(fn, "rb");
	if(fp == NULL)	{
		printf("Cannot open file: %s\n", fn);
		return -1;
	}

	fread(&sig, 3, 1, fp);
	if(strncmp(sig, "LJM", 3))	{
		printf("File %s is not a valid .ljm map!\n", fn);
		return -1;
	}

	fread(&version, sizeof(uint32), 1, fp);
	fread(&dim, sizeof(uint8), 1, fp);
	fread(&tilecount, sizeof(uint32), 1, fp);
	fread(&map_x, sizeof(uint32), 1, fp);
	fread(&map_y, sizeof(uint32), 1, fp);

	if(version != 1)	{
		fread(&eventcount, sizeof(uint32), 1, fp);
		fread(&animcount, sizeof(uint32), 1, fp);
	}

	tiles = (tile_dat_t *) malloc(sizeof(tile_dat_t) * tilecount);
	fread(tiles, sizeof(tile_dat_t) * tilecount, 1, fp);


	if(version != 1)	{
		event = (ljm_event_t *) malloc(sizeof(ljm_event_t) * eventcount);
		anim = (ljm_anim_t *) malloc(sizeof(ljm_anim_t) * animcount);
		fread(event, sizeof(ljm_event_t) * eventcount, 1, fp);
		for(i = 0; i < animcount; i++)	{
			fread(&anim[i].count, sizeof(uint32), 1, fp);
			anim[i].seq = (uint32 *) malloc(sizeof(uint32) * anim[i].count);
			fread(anim[i].seq, sizeof(uint32) * anim[i].count, 1, fp);
			fread(&anim[i].delay, sizeof(uint8), 1, fp);
		}
	}

	tmp_txr = (char *) malloc(dim * dim * 2);
	tiles_txr = (plx_texture_t **) malloc(sizeof(plx_texture_t *) * tilecount);
	tile_map = (uint32 **) malloc(sizeof(uint32 *) * map_y);
	for(i = 0; i < tilecount; i++)	{
		fread(tmp_txr, dim * dim * 2, 1, fp);
		tiles_txr[i] = plx_txr_canvas(dim, dim, PVR_TXRFMT_RGB565);
		pvr_txr_load_ex(tmp_txr, tiles_txr[i]->ptr, dim, dim, PVR_TXRLOAD_16BPP);
	}

	for(i = 0; i < map_y; i++)	{
		tile_map[i] = (uint32 *) malloc(sizeof(uint32) * map_x);
		fread(tile_map[i], sizeof(uint32) * map_x, 1, fp);
	}

	fclose(fp);
	free(tmp_txr);

	starttime = timer_ms_gettime64();
	for(i = 0; i < animcount; i++)	{
		anim[i].sttime = starttime;
		anim[i].current = 0;
	}

	xoffset = 0;
	yoffset = 0;

	if(magnify > 1)	{
		dim *= magnify;
	}

	return 0;
}

int ljm_free()	{
	uint32 i;
	for(i = 0; i < tilecount; i++)	{
		plx_txr_destroy(tiles_txr[i]);
	}

	free(tiles);
	free(tiles_txr);

	for(i = 0; i < map_y; i++)	{
		free(tile_map[i]);
	}
	free(tile_map);

	if(version != 1)	{
		for(i = 0; i < animcount; i++)	{
			free(anim[i].seq);
		}
		free(anim);
		free(event);
	}

	xoffset = 0;
	yoffset = 0;
	dim = 0;
	eventcount = 0;
	animcount = 0;
	tilecount = 0;
	map_x = 0;
	map_y = 0;
	version = 0;
	return 0;
}

/* These functions now return pointers, not a regular struct! */

/* Get tile data based on block numbers */
tile_dat_t * ljm_get_data_blk(uint32 blknum)	{
	if(blknum < 2147483649UL)	{
		return &tiles[blknum];
	}
	else	{
		return &tiles[anim[blknum - 2147483649UL].seq[0]];
	}
}

/* Get the data of a tile based on its x and y coordinates (in pixels) */
tile_dat_t * ljm_get_data_pixels(uint32 x, uint32 y)	{
	if(tile_map[y / dim][x / dim] < 2147483649UL)	{
		return &tiles[tile_map[y / dim][x / dim]];
	}
	else	{
		return &tiles[anim[tile_map[y / dim][x / dim] - 2147483649UL].seq[0]];
	}
}

/* Get tile data based on x/y coordinates in blocks */
tile_dat_t * ljm_get_data_blocks(uint32 x, uint32 y)	{
	if(tile_map[y][x] < 2147483649UL)	{
		return &tiles[tile_map[y][x]];
	}
	else	{
		return & tiles[anim[tile_map[y][x] - 2147483649UL].seq[0]];
	}
}

/* Scrolling functions -- We really don't need these, but in case I ever make the variables static, here they are */
void ljm_set_xoffset(uint32 xoff)	{
	xoffset = xoff;
}

void ljm_set_yoffset(uint32 yoff)	{
	yoffset = yoff;
}

void ljm_map_scroll(uint32 xpos, uint32 ypos)	{
	xoffset = xpos;
	yoffset = ypos;
}

/* Returns basic info about the current map -- Once again, not really needed*/
ljm_info_t ljm_map_info()	{
	ljm_info_t info;
	info.dim = dim;
	info.xblocks = map_x;
	info.yblocks = map_y;
	info.tilecount = tilecount;
	info.version = version;

	return info;
}

/* Here we assume that the appropriate list has been opened, and the scene begun, the user must also end the scene */
void ljm_draw_bg(plx_dr_state_t * state)	{
	uint32 i, j, animnum, animcur;

	for(i = 0; i + yoffset < map_y; i++)	{
		for(j = 0; j + xoffset < map_x; j++)	{
			if(i * dim < 480 && j * dim < 640)	{
				if(tile_map[i + yoffset][j + xoffset] < 2147483649UL)	{
					plx_txr_send_hdr(tiles_txr[tile_map[i + yoffset][j + xoffset]], PLX_LIST_OP_POLY, 0);
					plx_spr_fnd(state, dim, dim, dim * j + (dim / 2), dim * i + (dim / 2), 1.0f, 1.0f, 1, 1, 1);
				}
				else	{
					animnum = tile_map[i + yoffset][j + xoffset] - 2147483649UL;
					animcur = anim[animnum].current;
					plx_txr_send_hdr(tiles_txr[anim[animnum].seq[animcur]], PLX_LIST_OP_POLY, 0);
					plx_spr_fnd(state, dim, dim, dim * j + (dim / 2), dim * i + (dim / 2), 1.0f, 1.0f, 1, 1, 1);
				}
			}
		}
	}
}

/* Increment any animations as appropriate */
void ljm_inc_anims()	{
	uint64 timer = timer_ms_gettime64();
	uint32 i;

	for(i = 0; i < animcount; i++)	{
		if(anim[i].sttime + (anim[i].delay * 100) <= timer)	{
			anim[i].sttime = timer;
			if(anim[i].current + 1 == anim[i].count)
				anim[i].current = 0;
			else
				anim[i].current++;
		}
	}
}

/* Scan for any event at a particular location, in blocks */
ljm_event_t * ljm_scan_events(uint32 x, uint32 y)	{
	uint32 i;
	ljm_event_t * eventfound = NULL;

	for(i = 0; i < eventcount; i++)	{
		if(event[i].evx == x && event[i].evy == y)	{
			eventfound = &event[i];
			break;
		}
	}

	return eventfound;
}

/* Scan for a certain type of event based on location, in blocks */
ljm_event_t * ljm_scan_events_type(uint32 x, uint32 y, uint8 type)	{
	uint32 i;
	ljm_event_t * eventfound = NULL;
	for(i = 0; i < eventcount; i++)	{
		if(event[i].evx == x && event[i].evy == y && event[i].evtype == type)	{
			eventfound = &event[i];
			break;
		}
	}

	return eventfound;
}
