/*	tilemap.c -- ljsdcdev
	.ljm Interface
	ljsdcdev
	Copyright (C) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljsdcdev.h"
#include "ljm.h"

char map_fn[30];
uint32 posx = 0, posy = 0;
static int sposx = 0, sposy = 0;
static int tm_done = 0;
int areanum;
static int areatot = 0;
static int npc_tot = 0;
static int msg_tot = 0;
static int warpcount = 0;
static int tcount = 0;
static int icount = 0;
static int scount = 0;
static int inncount = 0;
int enemies_used = 0;
static int num_en;
static int en_num;
static stat_data_t blank;
area_t * cur_area;
area_t areas[10];
static plx_texture_t * open_box = NULL;
static plx_texture_t * closed_box = NULL;

static const char msg_bleh[] = "Nothing interesting here...";
static char msg_treas[255];

/* For the new random battle determiner */
static int area_freq, area_min, steps;
static int area_maxenem, area_minenem;

static stat_data_t en_to_stat(en_desc_t * en)	{
	char realfn[256];
	stat_data_t enemy_out;

	strcpy(enemy_out.name, en->name);
	enemy_out.stats.attack = en->attack;
	enemy_out.stats.defense = en->defense;
	enemy_out.stats.speed = en->speed;
	enemy_out.stats.mdef = en->mdef;
	enemy_out.stats.matt = en->matt;
	enemy_out.stats.accuracy = en->accuracy;
	enemy_out.stats.hp = en->hp;
	enemy_out.stats.hp_rem = en->hp;
	enemy_out.stats.mp = en->mp;
	enemy_out.stats.mp_rem = en->mp;

	enemy_out.magic.mag_lev1[0] = en->magics[0];
	enemy_out.magic.mag_lev1[1] = en->magics[1];
	enemy_out.magic.mag_lev1[2] = en->magics[2];
	enemy_out.magic.mag_lev1[3] = en->magics[3];


	enemy_out.stats.element = en->element;
	enemy_out.stats.sxp = en->sxp;
	enemy_out.stats.exp = en->exp;
	enemy_out.stats.gold = en->gold;
	enemy_out.ally = -1;

	sprintf(realfn, "/cd/images/%s", en->fn);
	enemy_out.images.splayer = plx_txr_load(realfn, 1, PVR_TXRLOAD_16BPP);

	return enemy_out;
}

static int map_open = 0, npc_open = 0, sh_open = 0;

static void tm_xml_start(void *userData, const char *name, const char **atts)	{
	int i, j, k;
	char area_char[3];
	char map_char[3];
	uint16 enemy_num = 0;
	char realfn[256];
	sprintf(area_char, "%d", areanum);
	sprintf(map_char, "%d", mapnum);

	if(!strcmp(name, "MAP"))	{
		for(i = 0; atts[i]; i += 2)	{
			if(!strcmp("NUMBER", atts[i]))	{
				if(!strcmp(map_char, atts[i + 1]))	{
					map_open = 1;
					for(j = 0; atts[j]; j += 2)	{
						if(!strcmp(atts[j], "FN"))	{
							strcpy(map_fn, atts[j +1]);
						}
					}
				}
			}
		}
	}
	else if(!strcmp(name, "AREA") && map_open == 1)	{
		for(i = 0; atts[i]; i += 2)	{
			if(!strcmp("NUMBER", atts[i]))	{
				for(j = 0; atts[j]; j += 2)	{
					if(!strcmp(atts[j], "X1"))	{
						areas[areatot].x1 = atoi(atts[j + 1]);
					}
					else if(!strcmp(atts[j], "X2"))	{
						areas[areatot].x2 = atoi(atts[j + 1]);
					}
					else if(!strcmp(atts[j], "Y1"))	{
						areas[areatot].y1 = atoi(atts[j + 1]);
					}
					else if(!strcmp(atts[j], "Y2"))	{
						areas[areatot].y2 = atoi(atts[j + 1]);
					}
					else if(!strcmp(atts[j], "MINEN"))	{
						areas[areatot].minen = atoi(atts[j + 1]);
					}
					else if(!strcmp(atts[j], "MAXEN"))	{
						areas[areatot].maxen = atoi(atts[j + 1]);
					}
					else if(!strcmp(atts[j], "FREQ"))	{
						areas[areatot].freq = atoi(atts[j + 1]);
					}
					else if(!strcmp(atts[j], "MIN"))	{
						areas[areatot].minst = atoi(atts[j + 1]);
					}
				}
			}
		}
	}
	else if(!strcmp(name, "ENEMY") && map_open == 1)	{
		for(j = 0; atts[j]; j += 2)	{
			if(!strcmp(atts[j], "NUM"))	{
				enemy_num = atoi(atts[j + 1]);
			}
		}
		for(j = 0; atts[j]; j += 2)	{
			if(!strcmp(atts[j], "CHANCE"))	{
				k = atoi(atts[j + 1]) + enemies_used;
				areas[areatot].enemy[k - 1] = en_to_stat(&en_desc[enemy_num]);
				k--;
				for(; k > enemies_used; k--)	{
					areas[areatot].enemy[k - 1] = en_to_stat(&en_desc[enemy_num]);
				}
				enemies_used += atoi(atts[j + 1]);
			}
		}
	}
	else if(!strcmp(name, "NPC") && map_open == 1)	{
		for(j = 0; atts[j]; j += 2)	{
			if(!strcmp(atts[j], "X"))	{
				areas[areatot].npcs[npc_tot].x = atoi(atts[j + 1]);
				areas[areatot].npcs[npc_tot].curx = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "Y"))	{
				areas[areatot].npcs[npc_tot].y = atoi(atts[j + 1]);
				areas[areatot].npcs[npc_tot].cury = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "MOV"))	{
				areas[areatot].npcs[npc_tot].mov = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "FREQ"))	{
				areas[areatot].npcs[npc_tot].freq = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "IMG"))	{
				sprintf(realfn, "/cd/images/%s", atts[j + 1]);
				areas[areatot].npcs[npc_tot].txr = plx_txr_load(realfn, 1, PVR_TXRLOAD_16BPP);
			}
			else if(!strcmp(atts[j], "MSGS"))	{
				areas[areatot].npcs[npc_tot].msg_count = atoi(atts[j + 1]);
				areas[areatot].npcs[npc_tot].msg_num = -1;
				areas[areatot].npcs[npc_tot].msgs = (char **) malloc(sizeof(char *) * areas[areatot].npcs[npc_tot].msg_count);
			}
		}
		areas[areatot].npcs[npc_tot].time_move = timer_ms_gettime64() + (areas[areatot].npcs[npc_tot].freq * 100);
		npc_open = 1;
	}
	else if(!strcmp(name, "MSG") && npc_open == 1)	{
		if(!strcmp(atts[0], "TXT"))	{
			areas[areatot].npcs[npc_tot].msgs[msg_tot] = (char *) malloc(sizeof(char) * strlen(atts[1]) + 1);
			strcpy(areas[areatot].npcs[npc_tot].msgs[msg_tot], atts[1]);
		}
	}
	else if(!strcmp(name, "PROPERTIES") && map_open == 1)	{
		for(j = 0; atts[j]; j+= 2)	{
			if(!strcmp(atts[j], "WARPS"))	{
				areas[areatot].warp_count = atoi(atts[j + 1]);
				areas[areatot].warps = (warp_t *) malloc(sizeof(warp_t) * areas[areatot].warp_count);
			}
			else if(!strcmp(atts[j], "NPCS"))	{
				areas[areatot].npc_count = atoi(atts[j + 1]);
				areas[areatot].npcs = (npc_t *) malloc(sizeof(npc_t) * areas[areatot].npc_count);
				for(i = 0; i < areas[areatot].npc_count; i++)	{
					areas[areatot].npcs[i].msg_count = 0;
					areas[areatot].npcs[i].msgs = NULL;
					areas[areatot].npcs[i].txr = NULL;
				}
			}
			else if(!strcmp(atts[j], "TREASURES"))	{
				areas[areatot].treas_count = atoi(atts[j + 1]);
				areas[areatot].treas = (treasure_t *) malloc(sizeof(treasure_t) * areas[areatot].treas_count);
			}
			else if(!strcmp(atts[j], "SHOPS"))	{
				areas[areatot].shopcount = atoi(atts[j + 1]);
				areas[areatot].shop = (shop_t *) malloc(sizeof(shop_t) * areas[areatot].shopcount);
			}
			else if(!strcmp(atts[j], "INNS"))	{
				areas[areatot].inncount = atoi(atts[j + 1]);
				areas[areatot].inn = (inn_t *) malloc(sizeof(inn_t) * areas[areatot].inncount);
			}
		}
	}
	else if(!strcmp(name, "WARP") && map_open == 1)	{
		for(j = 0; atts[j]; j += 2)	{
			if(!strcmp(atts[j], "X"))	{
				areas[areatot].warps[warpcount].x = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "Y"))	{
				areas[areatot].warps[warpcount].y = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "TO"))	{
				areas[areatot].warps[warpcount].map = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "TOX"))	{
				areas[areatot].warps[warpcount].tox = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "TOY"))	{
				areas[areatot].warps[warpcount].toy = atoi(atts[j + 1]);
			}
		}
	}
	else if(!strcmp(name, "TREASURE") && map_open == 1)	{
		for(j = 0; atts[j]; j += 2)	{
			if(!strcmp(atts[j], "X"))	{
				areas[areatot].treas[tcount].x = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "Y"))	{
				areas[areatot].treas[tcount].y = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "TYPE"))	{
				areas[areatot].treas[tcount].type = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "COUNT"))	{
				areas[areatot].treas[tcount].count = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "SWITCH"))	{
				areas[areatot].treas[tcount].switch_num = atoi(atts[j + 1]);
			}
		}
	}
	else if(!strcmp(name, "SHOP") && map_open == 1)	{
		for(j = 0; atts[j]; j += 2)	{
			if(!strcmp(atts[j], "NAME"))	{
				areas[areatot].shop[scount].name = (char *) malloc(strlen(atts[j + 1]) + 1);
				strcpy(areas[areatot].shop[scount].name, atts[j + 1]);
			}
			else if(!strcmp(atts[j], "X"))	{
				areas[areatot].shop[scount].x = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "Y"))	{
				areas[areatot].shop[scount].y = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "TYPE"))	{
				areas[areatot].shop[scount].type = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "COUNT"))	{
				areas[areatot].shop[scount].itemcount = atoi(atts[j + 1]);
				areas[areatot].shop[scount].items = (uint8 *) malloc(areas[areatot].shop[scount].itemcount);
			}
		}
		sh_open = 1;
	}
	else if(!strcmp(name, "ITEM") && sh_open == 1)	{
		areas[areatot].shop[scount].items[icount] = (uint8) atoi(atts[1]);
	}

	else if(!strcmp(name, "INN") && map_open == 1)	{
		for(j = 0; atts[j]; j += 2)	{
			if(!strcmp(atts[j], "NAME"))	{
				areas[areatot].inn[inncount].name = (char *) malloc(strlen(atts[j + 1]) + 1);
				strcpy(areas[areatot].inn[inncount].name, atts[j + 1]);
			}
			else if(!strcmp(atts[j], "X"))	{
				areas[areatot].inn[inncount].x = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "Y"))	{
				areas[areatot].inn[inncount].y = atoi(atts[j + 1]);
			}
			else if(!strcmp(atts[j], "COST"))	{
				areas[areatot].inn[inncount].cost = atoi(atts[j + 1]);
			}
		}
	}
}

static void tm_xml_end(void *userData, const char *name)	{
	if(!strcmp(name, "MAP"))	{
		map_open = 0;
	}
	else if(!strcmp(name, "AREA") && map_open == 1)	{
		areatot++;
	}
	else if(!strcmp(name, "NPC") && map_open == 1)	{
		npc_tot++;
		msg_tot = 0;
		npc_open = 0;
	}
	else if(!strcmp(name, "MSG") && npc_open == 1)	{
		msg_tot++;
	}
	else if(!strcmp(name, "WARP") && map_open == 1)	{
		warpcount++;
	}
	else if(!strcmp(name, "TREASURE") && map_open == 1)	{
		tcount++;
	}
	else if(!strcmp(name, "SHOP") && map_open == 1)	{
		scount++;
		sh_open = 0;
		icount = 0;
	}
	else if(!strcmp(name, "ITEM") && sh_open == 1)	{
		icount++;
	}
	else if(!strcmp(name, "INN") && map_open == 1)	{
		inncount++;
	}
}

/* Random battle determiner, as recomended by MrSiggler */
static void tilemap_battle()	{
	posx = (xoffset * dim) + sposx;
	posy = (yoffset * dim) + sposy;
	int i;
	steps--;

	if(steps == 0)	{
		do_battle();
  		srand(time(NULL));
		num_en = ljsdcdev_random(area_minenem, area_maxenem);
		for(i = 0; i <= 8; i++)	{
			enemies[i] = blank;
		}
		for(i = 0; i <= num_en; i++)	{
			en_num = ljsdcdev_random(0, 9);
			enemies[i] = areas[0].enemy[en_num];
		}
		steps = ljsdcdev_random(0, area_freq) + area_min;
	}
}

static inline int tilemap_is_visible(uint16 x, uint16 y)	{
	if(x < (xoffset * dim) || y < (yoffset * dim) || x > (xoffset * dim) + 640 || y > (yoffset * dim) + 480)	{
		return 0;
	}
	return 1;
}

static inline int tilemap_check_treas(uint16 x, uint16 y)	{
	int i;
	for(i = 0; i < cur_area->treas_count; i++)	{
		if(cur_area->treas[i].x == x && cur_area->treas[i].y == y)
			return -1;
	}
	return 0;
}

static inline int tilemap_check_shop(uint16 x, uint16 y)	{
	int i;
	for(i = 0; i < cur_area->shopcount; i++)	{
		if(cur_area->shop[i].x == x && cur_area->shop[i].y == y)
			return i + 1;
	}
	return 0;
}

static inline int tilemap_check_inn(uint16 x, uint16 y)	{
	int i;
	for(i = 0; i < cur_area->inncount; i++)	{
		if(cur_area->inn[i].x == x && cur_area->inn[i].y == y)
			return i + 1;
	}
	return 0;
}

static char * tilemap_open_treas(uint16 x, uint16 y)	{
	int i;
	for(i = 0; i < cur_area->treas_count; i++)	{
		if(cur_area->treas[i].x == x && cur_area->treas[i].y == y)	{
			if(cur_area->treas[i].type <= 255)	{
				if(switches[cur_area->treas[i].switch_num] == 0)	{
					misc_add_item(cur_area->treas[i].type, cur_area->treas[i].count);
					sprintf(msg_treas, "You got %d %s!", cur_area->treas[i].count, items[cur_area->treas[i].type].name);
					switches[cur_area->treas[i].switch_num] = 1;
				}
				else	{
					sprintf(msg_treas, "This box used to contain %d %s.", cur_area->treas[i].count, items[cur_area->treas[i].type].name);
				}
			}
			else if(cur_area->treas[i].type <= 511)	{
				if(switches[cur_area->treas[i].switch_num] == 0)	{
					misc_add_equip(cur_area->treas[i].type - 256);
					sprintf(msg_treas, "You got a %s!", equipment[cur_area->treas[i].type - 256].name);
					switches[cur_area->treas[i].switch_num] = 1;
				}
				else	{
					sprintf(msg_treas, "This box used to contain a %s.", equipment[cur_area->treas[i].type - 256].name);
				}
			}
			else if(cur_area->treas[i].type == 65535)	{
				if(switches[cur_area->treas[i].switch_num] == 0)	{
					gold += cur_area->treas[i].count;
					sprintf(msg_treas, "You recieved %d gold!", cur_area->treas[i].count);
					switches[cur_area->treas[i].switch_num] = 1;
				}
				else	{
					sprintf(msg_treas, "This box used to contain %d gold.", cur_area->treas[i].count);
				}
			}
			return msg_treas;
		}
	}
	return NULL;
}

static inline void tilemap_show_treas()	{
	int i;
	for(i = 0; i < cur_area->treas_count; i++)	{
		if(tilemap_is_visible(cur_area->treas[i].x * dim, cur_area->treas[i].y * dim))	{
			if(switches[cur_area->treas[i].switch_num] == 0)	{
				plx_txr_send_hdr(closed_box, PLX_LIST_TR_POLY, 0);
			}
			else	{
				plx_txr_send_hdr(open_box, PLX_LIST_TR_POLY, 0);
			}
			plx_spr_fnd(&dr_state, dim, dim, cur_area->treas[i].x * dim + (dim / 2) - (xoffset * dim), cur_area->treas[i].y * dim + (dim / 2) - (yoffset * dim), 1.0f, 1.0f, 1, 1, 1);
		}
	}
}

static inline void tilemap_show_npcs()	{
	int i;
	for(i = 0; i < cur_area->npc_count; i++)	{
		if(tilemap_is_visible(cur_area->npcs[i].curx * dim, cur_area->npcs[i].cury * dim))	{
			plx_txr_send_hdr(cur_area->npcs[i].txr, PLX_LIST_TR_POLY, 0);
			plx_spr_fnd(&dr_state, dim, dim, cur_area->npcs[i].curx * dim + (dim / 2) - (xoffset * dim), cur_area->npcs[i].cury * dim + (dim / 2) - (yoffset * dim), 1.0f, 1.0f, 1, 1, 1);
		}
	}
}

static void tilemap_move_npcs()	{
	int i;
	int direction;
	for(i = 0; i < cur_area->npc_count; i++)	{
		if(cur_area->npcs[i].time_move <= timer_ms_gettime64())	{
			direction = ljsdcdev_random(0, 5);
			if(direction == 0)	{
				if(cur_area->npcs[i].curx + 1 - cur_area->npcs[i].x <= cur_area->npcs[i].mov)	{
					if(cur_area->npcs[i].cury * dim != posy || (cur_area->npcs[i].curx + 1) * dim != posx)	{
						cur_area->npcs[i].curx++;
					}
				}
			}
			else if(direction == 1)	{
				if(cur_area->npcs[i].curx + 1 - cur_area->npcs[i].x >= 0 - cur_area->npcs[i].mov)	{
					if(cur_area->npcs[i].cury * dim != posy || (cur_area->npcs[i].curx - 1) * dim != posx)	{
						cur_area->npcs[i].curx--;
					}
				}
			}
			else if(direction == 2)	{
				if(cur_area->npcs[i].cury + 1 - cur_area->npcs[i].y <= cur_area->npcs[i].mov)	{
					if(cur_area->npcs[i].curx * dim != posx || (cur_area->npcs[i].cury + 1) * dim != posy)	{
						cur_area->npcs[i].cury++;
					}
				}
			}
			else if(direction == 3)	{
				if(cur_area->npcs[i].cury + 1 - cur_area->npcs[i].y >= 0 - cur_area->npcs[i].mov)	{
					if(cur_area->npcs[i].curx * dim != posx || (cur_area->npcs[i].cury -1) * dim != posy)	{
						cur_area->npcs[i].cury--;
					}
				}
			}
			cur_area->npcs[i].time_move = timer_ms_gettime64() + (cur_area->npcs[i].freq * 100);
		}
	}
}

static inline int tilemap_check_npcs(uint16 x, uint16 y)	{
	int i;
	for(i = 0; i < cur_area->npc_count; i++)	{
		if(cur_area->npcs[i].curx == x && cur_area->npcs[i].cury == y)
			return -1;
	}
	return 0;
}

static char * tilemap_check_npcmsg(uint16 x, uint16 y)	{
	int i;
	for(i = 0; i < cur_area->npc_count; i++)	{
		if(cur_area->npcs[i].curx == x && cur_area->npcs[i].cury == y)	{
			cur_area->npcs[i].msg_num++;
			if(cur_area->npcs[i].msg_num >= cur_area->npcs[i].msg_count)	{
				cur_area->npcs[i].msg_num = 0;
			}
			return cur_area->npcs[i].msgs[(int) cur_area->npcs[i].msg_num];
		}
	}
	return NULL;
}

static void tilemap_scroll()	{
	while(sposx > 480 && xoffset != map_x)	{
		sposx -= dim;
		xoffset++;
	}
	while(sposy > 320 && yoffset != map_y)	{
		sposy -= dim;
		yoffset++;
	}
	if(sposx < 160 && xoffset != 0)	{
		sposx += dim;
		xoffset--;
	}
	if(sposy < 160 && yoffset != 0)	{
		sposy += 32;
		yoffset--;
	}
}

void tilemap_init(uint32 x, uint32 y)	{
	char * buf;
	unsigned long size;
	char real_fn[50];
	FILE * fp;
	XML_Parser xmlp = XML_ParserCreate(NULL);
	int i;

	XML_SetElementHandler(xmlp, tm_xml_start, tm_xml_end);

	warpcount = 0;
	areatot = 0;
	npc_tot = 0;
	enemies_used = 0;
	num_en = 0;
	en_num = 0;
	scount = 0;
	icount = 0;
	tcount = 0;
	inncount = 0;

	memset(areas, 0, sizeof(area_t) * 10);
	cur_area = &areas[0];
	for(i = 0; i < 10; i++)	{
		areas[i].npcs = NULL;
		areas[i].npc_count = 0;
		areas[i].warps = NULL;
		areas[i].warp_count = 0;
		areas[i].treas = NULL;
		areas[i].treas_count = 0;
		areas[i].shop = NULL;
		areas[i].shopcount = 0;
		areas[i].inn = NULL;
		areas[i].inncount = 0;
	}

	if(open_box == NULL)	{
		open_box = plx_txr_load("/cd/images/treas_op.png", 1, PVR_TXRLOAD_16BPP);
	}

	if(closed_box == NULL)	{
		closed_box = plx_txr_load("/cd/images/treas_cl.png", 1, PVR_TXRLOAD_16BPP);
	}

	fp = fopen("/cd/data/areas.xml", "r");
	size = misc_get_filesize(fp);
	buf = (char *) malloc(size);
	fread(buf, size, 1, fp);
	fclose(fp);

	if (XML_Parse(xmlp, buf, size, 1) == XML_STATUS_ERROR) {
		printf("%s at line %d\n", XML_ErrorString(XML_GetErrorCode(xmlp)),
				XML_GetCurrentLineNumber(xmlp));
		return;
	}
	XML_ParserFree(xmlp);
	free(buf);

	sprintf(real_fn, "/cd/maps/%s", map_fn);
	ljm_load(real_fn, 2);

	posx = x;
	posy = y;
	sposx = x;
	sposy = y;

	tilemap_scroll();
	posx = (xoffset * dim) + sposx;
	posy = (yoffset * dim) + sposy;

	area_freq = areas[0].freq;
	area_min = areas[0].minst;
	area_maxenem = areas[0].maxen;
	srand(time(NULL));
	steps = ljsdcdev_random(0, area_freq) + area_min;
	num_en = ljsdcdev_random(area_minenem, area_maxenem);

	for(i = 0; i <= num_en; i++)	{
		en_num = ljsdcdev_random(0, 9);
		enemies[i] = areas[0].enemy[en_num];
	}
}

static warp_t * tilemap_check_warps()	{
	int i;
	for(i = 0; i < warpcount; i++)	{
		if(areas[0].warps[i].x == posx / dim && areas[0].warps[i].y == posy / dim)	{
			return &areas[0].warps[i];
		}
	}
	return NULL;
}

void tilemap_mainloop()	{
	int rand_needed = 0;
	cont_state_t * state;
	tile_dat_t * block;
	warp_t * warp;
	int direct1 = 0, direct2 = 0;
	uint64 time_since = timer_ms_gettime64();
	int msg_box_open = 0;
	int i, j, k;
	int npc_here = 0;
	int treas_here = 0;
	int shop_here = 0;
	int inn_here = 0;
	char * msg = NULL;
	int8 facing1 = 0, facing2 = 0;
	while(tm_done != 1)	{
		state = (cont_state_t *) maple_dev_status(maple_enum_type(0, MAPLE_FUNC_CONTROLLER));
		rand_needed = 0;
		ljm_inc_anims();
		if(msg_box_open == 0)	{
			tilemap_move_npcs();
		}
		plx_scene_begin();
		plx_list_begin(PLX_LIST_OP_POLY);
		ljm_draw_bg(&dr_state);
		plx_list_begin(PLX_LIST_TR_POLY);
		plx_txr_send_hdr(stats[0].images.splayer, PLX_LIST_TR_POLY, 0);
		plx_spr_fnd(&dr_state, dim, dim, sposx + (dim / 2), sposy + (dim / 2), 1.0f, 1.0f, 1, 1, 1);
		tilemap_show_npcs();
		tilemap_show_treas();
		if(msg_box_open == 1)	{
			msg_box_plain(msg, 0, 0, 255);
		}
		plx_scene_end();

		if(state != NULL)	{
			if(state->buttons & CONT_START && time_since + 200 < timer_ms_gettime64())	{
				menu_do_menu();
				time_since = timer_ms_gettime64();
				pvr_set_bg_color(0, 0, 0);
			}

			if(state->buttons & CONT_A && time_since + 200 < timer_ms_gettime64())	{
				if(msg_box_open == 0)	{
					msg = tilemap_check_npcmsg(posx / dim + facing1, posy / dim + facing2);
					if(msg == NULL)	{
						msg = tilemap_open_treas(posx / dim + facing1, posy / dim + facing2);
					}
					if(msg == NULL)	{
						msg = (char *) msg_bleh;
					}
					msg_box_open = 1;
				}
				else if(msg_box_open == 1)	{
					msg_box_open = 0;
				}
				time_since = timer_ms_gettime64();
			}
		}

		if(msg_box_open == 0)	{
			if(state != NULL)	{
				if(state->buttons & CONT_DPAD_UP && posy != 0 && time_since + 200 < timer_ms_gettime64())	{
					block = ljm_get_data_pixels(posx, posy - dim);
					npc_here = tilemap_check_npcs(posx / dim, posy / dim - 1);
					treas_here = tilemap_check_treas(posx / dim, posy / dim - 1);
					shop_here = tilemap_check_shop(posx / dim, posy / dim - 1);
					inn_here = tilemap_check_inn(posx / dim, posy / dim - 1);
					facing2 = -1;
					facing1 = 0;
					if(shop_here != 0)	{
						shop_open(cur_area, shop_here - 1);
						thd_sleep(200);
						shop_here = 0;
						continue;
					}
					if(inn_here != 0)	{
						inn_open(cur_area, inn_here - 1);
						thd_sleep(200);
						inn_here = 0;
						continue;
					}
					if(block->user4 == 0 && npc_here == 0 && treas_here == 0)	{
						direct1 = 1;
						rand_needed = 1;
					}
					else	{
						direct1 = direct2 = 0;
						rand_needed = 0;
						continue;
					}
				}
				if(state->buttons & CONT_DPAD_DOWN && posy != map_y * dim - dim && time_since + 200 < timer_ms_gettime64())	{
					block = ljm_get_data_pixels(posx, posy + dim);
					npc_here = tilemap_check_npcs(posx / dim, posy / dim + 1);
					treas_here = tilemap_check_treas(posx / dim, posy / dim + 1);
					shop_here = tilemap_check_shop(posx / dim, posy / dim + 1);
					inn_here = tilemap_check_inn(posx / dim, posy / dim + 1);
					facing2 = 1;
					facing1 = 0;
					if(shop_here != 0)	{
						shop_open(cur_area, shop_here - 1);
						thd_sleep(200);
						shop_here = 0;
						continue;
					}
					if(inn_here != 0)	{
						inn_open(cur_area, inn_here - 1);
						thd_sleep(200);
						inn_here = 0;
						continue;
					}
					if(block->user4 == 0 && npc_here == 0 && treas_here == 0)	{
						direct1 = 2;
						rand_needed = 1;
					}
					else	{
						direct1 = direct2 = 0;
						rand_needed = 0;
						continue;
					}
				}
				if(state->buttons & CONT_DPAD_LEFT && posx != 0 && time_since + 200 < timer_ms_gettime64())	{
					if(direct1 == 1)	{
						block = ljm_get_data_pixels(posx - dim, posy - dim);
						npc_here = tilemap_check_npcs(posx / dim - 1, posy / dim - 1);
						treas_here = tilemap_check_treas(posx / dim - 1, posy / dim - 1);
						shop_here = tilemap_check_shop(posx / dim - 1, posy / dim - 1);
						inn_here = tilemap_check_inn(posx / dim - 1, posy / dim - 1);
					}
					else if(direct1 == 2)	{
						block = ljm_get_data_pixels(posx - dim, posy + dim);
						npc_here = tilemap_check_npcs(posx / dim - 1, posy / dim + 1);
						treas_here = tilemap_check_treas(posx / dim - 1, posy / dim + 1);
						shop_here = tilemap_check_shop(posx / dim - 1, posy / dim + 1);
						inn_here = tilemap_check_inn(posx / dim - 1, posy / dim + 1);
					}
					else	{
						block = ljm_get_data_pixels(posx - dim, posy);
						npc_here = tilemap_check_npcs(posx / dim - 1, posy / dim);
						treas_here = tilemap_check_treas(posx / dim - 1, posy / dim);
						shop_here = tilemap_check_shop(posx / dim - 1, posy / dim);
						inn_here = tilemap_check_inn(posx / dim - 1, posy / dim);
					}
					facing1 = -1;
					facing2 = 0;

					if(shop_here != 0)	{
						shop_open(cur_area, shop_here - 1);
						thd_sleep(200);
						shop_here = 0;
						continue;
					}
					if(inn_here != 0)	{
						inn_open(cur_area, inn_here - 1);
						thd_sleep(200);
						inn_here = 0;
						continue;
					}
					if(block->user4 == 0 && npc_here == 0 && treas_here == 0)	{
						direct2 = 1;
						rand_needed = 1;
					}
					else	{
						direct1 = direct2 = 0;
						rand_needed = 0;
						continue;
					}
				}
				if(state->buttons & CONT_DPAD_RIGHT && posx != map_x * dim - dim && time_since + 200 < timer_ms_gettime64())	{
					if(direct1 == 1)	{
						block = ljm_get_data_pixels(posx + dim, posy - dim);
						npc_here = tilemap_check_npcs(posx / dim + 1, posy / dim - 1);
						treas_here = tilemap_check_treas(posx / dim + 1, posy / dim - 1);
						shop_here = tilemap_check_shop(posx / dim + 1, posy / dim - 1);
						inn_here = tilemap_check_inn(posx / dim + 1, posy / dim - 1);
					}
					else if(direct1 == 2)	{
						block = ljm_get_data_pixels(posx + dim, posy + dim);
						npc_here = tilemap_check_npcs(posx / dim + 1, posy / dim + 1);
						treas_here = tilemap_check_treas(posx / dim + 1, posy / dim + 1);
						shop_here = tilemap_check_shop(posx / dim + 1, posy / dim);
						inn_here = tilemap_check_inn(posx / dim + 1, posy / dim);
					}
					else	{
						block = ljm_get_data_pixels(posx + dim, posy);
						npc_here = tilemap_check_npcs(posx / dim + 1, posy / dim);
						treas_here = tilemap_check_treas(posx / dim + 1, posy / dim);
						shop_here = tilemap_check_shop(posx / dim + 1, posy / dim);
						inn_here = tilemap_check_inn(posx / dim + 1, posy / dim);
					}
					facing1 = 1;
					facing2 = 0;

					if(shop_here != 0)	{
						shop_open(cur_area, shop_here - 1);
						thd_sleep(200);
						shop_here = 0;
						continue;
					}
					if(inn_here != 0)	{
						inn_open(cur_area, inn_here - 1);
						thd_sleep(200);
						inn_here = 0;
						continue;
					}
					if(block->user4 == 0 && npc_here == 0 && treas_here == 0)	{
						direct2 = 2;
						rand_needed = 1;
					}
					else	{
						direct1 = direct2 = 0;
						rand_needed = 0;
						continue;
					}
				}
			}
		}

		if(rand_needed == 1)	{
			if(direct1 == 1)	{
				sposy -= dim;
				posy -= dim;
			}
			else if(direct1 == 2)	{
				sposy += dim;
				posy += dim;
			}
			if(direct2 == 1)	{
				sposx -= dim;
				posx -= dim;
			}
			else if(direct2 == 2)	{
				sposx += dim;
				posx += dim;
			}
			direct1 = direct2 = 0;
			time_since = timer_ms_gettime64();
			warp = tilemap_check_warps();
			if(warp != NULL)	{
				posx = warp->tox * dim;
				posy = warp->toy * dim;
				mapnum = warp->map;

				for(i = 0; i < 10; i++)	{
					if(areas[i].warps != NULL)
						free(areas[i].warps);
					if(areas[i].enemy != NULL)	{
						for(j = 0; j < 10; j++)	{
							if(areas[i].enemy[j].images.splayer != NULL)	{
								plx_txr_destroy(areas[i].enemy[j].images.splayer);
								areas[i].enemy[j].images.splayer = NULL;
							}
						}
					}
					if(areas[i].npcs != NULL)	{
						for(j = 0; j < areas[i].npc_count; j++)	{
							if(areas[i].npcs[j].msgs != NULL)	{
								for(k = 0; k < areas[i].npcs[j].msg_count; k++)	{
									free(areas[i].npcs[j].msgs[k]);
								}
								free(areas[i].npcs[j].msgs);
							}
							if(areas[i].npcs[j].txr != NULL)	{
								plx_txr_destroy(areas[i].npcs[j].txr);
							}
						}
						free(areas[i].npcs);
					}
					if(areas[i].treas != NULL)	{
						free(areas[i].treas);
					}
					if(areas[i].shop != NULL)	{
						for(j = 0; j < areas[i].shopcount; j++)	{
							free(areas[i].shop[j].items);
							free(areas[i].shop[j].name);
						}
						free(areas[i].shop);
					}
					if(areas[i].inn != NULL)	{
						for(j = 0; j < areas[i].inncount; j++)	{
							free(areas[i].inn[j].name);
						}
						free(areas[i].inn);
					}
				}
				ljm_free();
				tilemap_init(posx, posy);
			}
			tilemap_battle();
			tilemap_scroll();
			misc_update_timer();
		}
	}
}

void do_tilemap()	{
	mapnum = 0;
	tilemap_init(0, 0);
	misc_start_timer();
	tilemap_mainloop();
}
