/*	vmusave.c	-- ljsdcdev
	Dreamcast specific saving routines
	ljsdcdev
	Copyright (C) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljsdcdev.h"

static int load_lji(const char * fn, char * data, char * pal, int invert_y)	{
	FILE * fp;
	char sig[3];
	uint8 ver;
	uint8 * data2 = (uint8 *) malloc(512);
	int i;

	fp = fopen(fn, "rb");
	if(fp == NULL)	{
		printf("Error opening .lji file: %s\n", fn);
		return -1;
	}

	fread(sig, 3, 1, fp);
	if(strncmp(sig, "LJI", 3))	{
		printf("Invalid signature on .lji file\n");
		return -1;
	}

	fread(&ver, 1, 1, fp);
	if(ver != 1)	{
		printf("Invalid .lji version: %d\n", ver);
		return -1;
	}

	fread(pal, 32, 1, fp);
	fread(data2, 512, 1, fp);

	if(invert_y == 1)	{
		for(i = 0; i < 32; i++)	{
			memcpy(data + i * 16, data2 + ((32 - i) * 16), 16);
		}
	}

	else	{
		memcpy(data, data2, 512);
	}

	free(data2);
	fclose(fp);
	return 0;
}

void save_data(save_data_t * save, const char * fn)	{
	uint8 * data;
	file_t fd;
	vmu_pkg_t pkg;
	uint8 * pkg_out;
	int pkg_size;
	uint8 * comp;
	uint32 len;
	char * img_data = (char *) malloc(512);
	char * img_pal = (char *) malloc(32);

	data = (uint8 *) malloc(sizeof(save_data_t));
	memcpy(data, save, sizeof(save_data_t));

	len = sizeof(save_data_t);
	comp = (uint8 *) malloc(len);

	compress2(comp, &len, data, sizeof(save_data_t), 9);

	load_lji("/cd/data/vmuicon.lji", img_data, img_pal, 1);

	strcpy(pkg.desc_short, "ljsdcdev");
	strcpy(pkg.desc_long, "ljsdcdev Demo 6");
	strcpy(pkg.app_id, "ljsdcdev");
	pkg.icon_cnt = 1;
	pkg.icon_anim_speed = 0;
	memcpy(pkg.icon_pal, img_pal, 32);
	pkg.icon_data = img_data;
	pkg.eyecatch_type = VMUPKG_EC_NONE;
	pkg.data_len = len;
	pkg.data = comp;

	vmu_pkg_build(&pkg, &pkg_out, &pkg_size);

	fs_unlink(fn);

	fd = fs_open(fn, O_WRONLY);
	fs_write(fd, pkg_out, pkg_size);

	fs_close(fd);
	free(pkg_out);
	free(data);
	free(comp);
	free(img_data);
	free(img_pal);
}

static int read_data(save_data_t * save, const char * fn)	{
	uint8 * raw_data;
	uint32 raw_size;
	uint8 * comp_data;
	uint32 comp_size;
	uint8 * real_data;
	uint32 real_size;
	vmu_pkg_t pkg;
	file_t fd;

	fd = fs_open(fn, O_RDONLY);
	if(fd == 0)	{
		printf("No Save File Found\n");
		return -1;
	}

	raw_size = fs_total(fd);

	raw_data = (uint8 *) malloc(raw_size);
	fs_read(fd, raw_data, raw_size);

	vmu_pkg_parse(raw_data, &pkg);
	comp_size = pkg.data_len;
	comp_data = (uint8 *) malloc(comp_size);
	memcpy(comp_data, pkg.data, comp_size);
	real_size = sizeof(save_data_t);
	real_data = (uint8 *) malloc(real_size);

	uncompress(real_data, &real_size, comp_data, comp_size);

	memcpy(save, real_data, real_size);

	fs_close(fd);
	free(raw_data);
	free(comp_data);
	free(real_data);

	return 0;
}

static void __scan_vmus(uint8 vmus[])	{
	maple_device_t * device = NULL;
	int i, j;

	for(i = 0; i < 4; i++)	{
		for(j = 1; j < 3; j++)	{
			device = maple_enum_dev(i, j);
			if(device != NULL)	{
				if(device->info.functions & MAPLE_FUNC_MEMCARD)	{
					vmus[(j - 1) + (2 * i)] = 1;
				}
				else	{
					vmus[(j - 1) + (2 * i)] = 0;
				}
			}
			else	{
				vmus[(j - 1) + (2 * i)] = 0;
			}
		}
	}
}

int vmu_select(save_data_t * save, int mode)	{
	uint8 vmus[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	cont_state_t * state;
	uint8 port = 0, slot = 0;
	uint64 time_since = timer_ms_gettime64();
	char fn[50];
	int i;
	int done = 1;
	plx_texture_t * txr = plx_txr_load("/cd/images/vmu.png", 1, PVR_TXRLOAD_16BPP);
	pvr_poly_hdr_t hdr;
	pvr_poly_cxt_t cxt;
	pvr_vertex_t vert;

	pvr_poly_cxt_col(&cxt, PVR_LIST_TR_POLY);
	pvr_poly_compile(&hdr, &cxt);

	while(1)	{
		__scan_vmus(vmus);
		state = (cont_state_t *) maple_dev_status(maple_enum_type(0, MAPLE_FUNC_CONTROLLER));

		if(state != NULL)	{
			if(state->buttons & CONT_A && time_since + 200 < timer_ms_gettime64())	{
				if(vmus[(port * 2) + (slot)] == 1)	{
					break;
				}
			}

			if(state->buttons & CONT_B)	{
				done = 0;
				break;
			}

			if((state->buttons & CONT_DPAD_DOWN || state->buttons & CONT_DPAD_UP) && time_since + 200 < timer_ms_gettime64())	{
				if(slot == 0)	{
					slot++;
				}
				else	{
					slot--;
				}
				time_since = timer_ms_gettime64();
			}

			if(state->buttons & CONT_DPAD_LEFT && time_since + 200 < timer_ms_gettime64())	{
				if(port == 0)	{
					port = 3;
				}
				else	{
					port--;
				}
				time_since = timer_ms_gettime64();
			}

			if(state->buttons & CONT_DPAD_RIGHT && time_since + 200 < timer_ms_gettime64())	{
				if(port == 3)	{
					port = 0;
				}
				else	{
					port++;
				}
				time_since = timer_ms_gettime64();
			}
		}

		plx_scene_begin();
		plx_list_begin(PLX_LIST_OP_POLY);

		ljsdcdev_hline_tr(128, 512, 32, 255, 255, 255, 255);
		ljsdcdev_hline_tr(128, 512, 96, 255, 255, 255, 255);
		ljsdcdev_hline_tr(80, 560, 368, 255, 255, 255, 255);
		ljsdcdev_hline_tr(80, 560, 416, 255, 255, 255, 255);

		ljsdcdev_vline_tr(112, 48, 80, 255, 255, 255, 255);
		ljsdcdev_vline_tr(528, 48, 80, 255, 255, 255, 255);
		ljsdcdev_vline_tr(64, 384, 400, 255, 255, 255, 255);
		ljsdcdev_vline_tr(576, 384, 400, 255, 255, 255, 255);

		ljsdcdev_line_tr(112, 48, 128, 32, 255, 255, 255, 255);
		ljsdcdev_line_tr(64, 384, 80, 368, 255, 255, 255, 255);
		ljsdcdev_line_tr(512, 96, 528, 80, 255, 255, 255, 255);
		ljsdcdev_line_tr(560, 416, 576, 400, 255, 255, 255, 255);

		ljsdcdev_line_tr(128, 96, 112, 80, 255, 255, 255, 255);
		ljsdcdev_line_tr(80, 416, 64, 400, 255, 255, 255, 255);
		ljsdcdev_line_tr(528, 48, 512, 32, 255, 255, 255, 255);
		ljsdcdev_line_tr(576, 384, 560, 368, 255, 255, 255, 255);

		plx_list_begin(PLX_LIST_TR_POLY);

		pvr_prim(&hdr, sizeof(hdr));

		vert.flags = PVR_CMD_VERTEX;
		vert.x = 48 + (port * 128);
		vert.y = 240 + (slot * 128);
		vert.z = 1.0f;
		vert.u = 0.0f;
		vert.v = 0.0f;
		vert.oargb = 0;
		vert.argb = PVR_PACK_COLOR(0.5f, 0.0, 0.8f, 1.0f);
		pvr_prim(&vert, sizeof(vert));

		vert.y = 112 + (slot * 128);
		pvr_prim(&vert, sizeof(vert));

		vert.x = 176 + (port * 128);
		vert.y = 240 + (slot * 128);
		pvr_prim(&vert, sizeof(vert));

		vert.flags = PVR_CMD_VERTEX_EOL;
		vert.y = 112 + (slot * 128);
		pvr_prim(&vert, sizeof(vert));

		plx_fcxt_setsize(fcxt_ax, 24.0f);
		ljsdcdev_text(fcxt_ax, "Select a Memory Card", 144, 72, 255, 255, 255, 255);
		plx_fcxt_setsize(fcxt_ax, 16.0f);
		ljsdcdev_text(fcxt_ax, "A to Continue", 112, 400, 255, 255, 255, 255);
		ljsdcdev_text(fcxt_ax, "B to Cancel", 384, 400, 255, 255, 255, 255);

		plx_txr_send_hdr(txr, PLX_LIST_TR_POLY, 0);
		for(i = 0; i < 8; i++)	{
			if(vmus[i] == 1)
				plx_spr_fnd(&dr_state, 96, 96, ((i / 2) * 128) + 112, ((i % 2) * 128) + 176, 5.0f, 1.0f, 1, 1, 1);
		}
		plx_scene_end();
	}

	plx_txr_destroy(txr);

	if(mode == 0 && done == 1)	{
		sprintf(fn, "/vmu/%c%d/ljsdcdev", (char) (port + 97), (slot + 1));
		save_data(save, fn);
		thd_sleep(200);
		return 0;
	}

	if(mode == 1 && done == 1)	{
		sprintf(fn, "/vmu/%c%d/ljsdcdev", (char) (port + 97), (slot + 1));
		thd_sleep(200);
		return read_data(save, fn);
	}

	thd_sleep(200);
	return -1;
}
